package com.iot.modbus_rtcp;

import cn.hutool.json.JSONUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeansException;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.scheduling.annotation.EnableScheduling;

@EnableScheduling
@SpringBootApplication(scanBasePackages = {"com.iot.modbus_rtcp"})
public class ModbusRtcpApplication implements ApplicationContextAware {

    private static final Logger logger = LoggerFactory.getLogger("com.iot.modbus_rtcp.rabbit");

    public static void main(String[] args) {
        SpringApplication.run(ModbusRtcpApplication.class, args);
    }

    // 为RabbitTemplate设置路由到队列失败时调用的方法
    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        //获取RabbitTemplate
        RabbitTemplate rabbitTemplate = applicationContext.getBean(RabbitTemplate.class);
        //设置ReturnCallback
        rabbitTemplate.setReturnsCallback((returnedMessage) -> {
            logger.info("消息发送失败,应答码{},原因{},交换机{},路由键{},消息{}", returnedMessage.getReplyCode(),
                    returnedMessage.getReplyText(), returnedMessage.getExchange(), returnedMessage.getRoutingKey(),
                    JSONUtil.toJsonStr(returnedMessage.getMessage()));
        });
    }

}
