package com.iot.modbus_rtcp.netty;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Slf4j
public class ChannelGroup {

    private final ConcurrentHashMap<String, SyncPriorityChannel> mChannelMap = new ConcurrentHashMap<>();

    public void put(String identity, SyncPriorityChannel channel) {
        this.mChannelMap.put(identity, channel);
    }

    public SyncPriorityChannel get(String identity) {
        return this.mChannelMap.get(identity);
    }

    public SyncPriorityChannel get(String ip, int port) {
        return this.mChannelMap.get(getKey(ip, port));
    }

    public void remove(String ip, int port) {
        this.mChannelMap.remove(getKey(ip, port));
    }

    public void remove(String identity) {
        this.mChannelMap.remove(identity);
    }

    public Set<String> onlineGateway() {
        return this.mChannelMap.keySet()
                .stream()
                .filter(identifier -> !StringUtils.contains(identifier, ":"))
                .collect(Collectors.toSet());
    }

    public void see() {
        log.info("当前连接有:{}", new ArrayList<>(this.mChannelMap.keySet()));
    }

    public static String getKey(String ip, int port) {
        return ip + ":" + port;
    }
}
