package com.iot.modbus_rtcp.utils;

import com.iot.modbus_rtcp.dto.CommandTypeComparable;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.Collection;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 13:43
 */
@Getter
public enum ModbusFunctionCodeEnums {

    READ_COILS(1, CommandTypeComparable.CommandType.COLLECTION, "读取线圈状态", "位", "读取一组逻辑线圈的当前状态(ON/OFF)"),
    READ_DISCRETE_INPUTS(2, CommandTypeComparable.CommandType.COLLECTION, "读取输入状态", "位", "读取一组开关输入的当前状态(ON/OFF)"),
    READ_HOLDING_REGISTERS(3, CommandTypeComparable.CommandType.COLLECTION, "读取保持寄存器", "整数,字符型,状态字,浮点型", "在一个或多个保持寄存器中取得当前的二进制值"),
    READ_INPUT_REGISTERS(4, CommandTypeComparable.CommandType.COLLECTION, "读取输入寄存器", "整数,状态字,浮点型", "在一个或多个输入寄存器中取得当前的二进制值"),
    WRITE_COIL(5, CommandTypeComparable.CommandType.CONTROL, "强置单线圈", "位", "强置一个逻辑线圈的通断状态"),
    WRITE_REGISTER(6, CommandTypeComparable.CommandType.CONTROL, "预置单寄存器", "整数,字符型,状态字,浮点型", "把具体二进值装入一个保持寄存器"),
    //    READ_EXCEPTION_STATUS(7, CommandTypeComparable.CommandType.COLLECTION, "读取异常状态", "", "取得8个内部线圈的通断状态，这8个线圈的地址由控制器决定，用户逻辑可以将这些线圈定义，以说明从机状态，短报文适宜于迅速读取状态"),
    //    READ_EXCEPTION_STATUS(8, CommandTypeComparable.CommandType.COLLECTION, "回送诊断校验", "", "把诊断校验报文送从机，以对通信处理进行评鉴"),
    WRITE_COILS(15, CommandTypeComparable.CommandType.CONTROL, "强置多线圈", "位", "强置一串连续逻辑线圈的通断"),
    WRITE_REGISTERS(16, CommandTypeComparable.CommandType.CONTROL, "预置多寄存器", "整数,字符型,状态字,浮点型", "把具体的二进制值装入一串连续的保持寄存器"),
//    REPORT_SLAVE_ID(17, CommandTypeComparable.CommandType.COLLECTION, "报告从机标识", "字符型", "可使主机判断编址从机的类型及该从机运行指示灯的状态"),
    ;
    private final int code;
    private final String name;
    private final String dataType;
    private final String description;
    private final CommandTypeComparable.CommandType commandType;

    ModbusFunctionCodeEnums(int code, CommandTypeComparable.CommandType commandType, String name, String dataType, String description) {
        this.code = code;
        this.name = name;
        this.dataType = dataType;
        this.commandType = commandType;
        this.description = description;
    }

    public String getFunctionCode() {
        return StringUtils.leftPad(Integer.toHexString(this.code), 2, "0");
    }

    public static Collection<ModbusFunctionCodeEnums> getAllControlFunctionCode() {
        return Arrays.stream(values())
                .filter(item -> Objects.equals(CommandTypeComparable.CommandType.CONTROL, item.getCommandType()))
                .toList();
    }

    public static Collection<ModbusFunctionCodeEnums> getAllCollectionFunctionCode() {
        return Arrays.stream(values())
                .filter(item -> Objects.equals(CommandTypeComparable.CommandType.COLLECTION, item.getCommandType()))
                .toList();
    }

    public static boolean isHandlerHexCode(String hexCode) {
        return Arrays.stream(values()).anyMatch(item -> StringUtils.equalsIgnoreCase(item.getFunctionCode(), hexCode));
    }

}
